import json
import shutil
import sys
from pathlib import Path

import click

DEFAULT_RADIOS_JSON = Path(__file__).parent / "data" / "radios.json"
CONFIG_DIR = Path.home() / ".config" / "radio"
RADIOS_JSON = CONFIG_DIR / "radios.json"


def load_radios():
    """Load radios from data file (after creating if it does not exists)."""
    if not RADIOS_JSON.exists():
        if not CONFIG_DIR.exists():
            CONFIG_DIR.mkdir()
        with DEFAULT_RADIOS_JSON.open() as f:
            save(json.load(f))
    with RADIOS_JSON.open() as f:
        radios = json.load(f)
    return radios


def save(radios):
    """Save radios to file (after ordering by radio ID)."""
    radios = {rid: radios[rid] for rid in sorted(radios)}
    with RADIOS_JSON.open("w") as f:
        json.dump(radios, f, indent=4)


def find_installed_player():
    """Find an installed player."""
    # find installed player
    if shutil.which("ffplay"):
        player = ["ffplay", "ffplay", "-nodisp", "-loglevel", "panic"]
    elif shutil.which("cvlc"):
        player = ["cvlc", "cvlc"]
    elif shutil.which("mplayer"):
        player = ["mplayer", "mplayer"]
    else:
        player = None
    return player


def play_url(url):
    """Play an audio streaming url."""
    player = find_installed_player()
    if player is None:
        click.secho("Player NOT found!", fg="red")
        click.secho("Try installing any of the following packages:")
        click.secho("\tffmpeg, vlc or mplayer", bold=True)
        sys.exit(1)
    # launch player
    shutil.os.execlp(*player, url)


def show_list(radios, urls=None, count=None, str_filter="", invert=False):
    if count:
        click.echo(len(radios))
        sys.exit()
    data = "{}{}{name}\n\t\t{url}" if urls else "{}{}{name}"
    for rid, info in radios.items():
        condition = str_filter in rid or str_filter in info["name"].lower()
        condition = not condition if invert else condition
        if condition:
            tab = "\t\t" if len(rid) < 8 else "\t"
            click.secho(data.format(rid, tab, **info))


@click.group(help="Just listen to the radio.")
@click.version_option()
def radio():
    pass


@radio.command(help="Add or update a radio information.", no_args_is_help=True)
@click.option("-n", "--name", required=True, help="Radio complete fancy name.")
@click.option("-u", "--url", required=True, help="Radio playable streaming url.")
@click.argument("radio_id", required=True)
def add(radio_id, name, url):
    radios = load_radios()
    radios[radio_id] = {"name": name, "url": url}
    save(radios)
    click.secho("{} ({}) was succesfully added/updated".format(name, radio_id))


@radio.command(help="Remove a radio information.", no_args_is_help=True)
@click.argument("radio_id", required=True)
def remove(radio_id):
    radios = load_radios()
    if radio_id in radios:
        radio = radios.pop(radio_id)
        save(radios)
        click.secho("{} ({}) was succesfully removed".format(radio["name"], radio_id))
    else:
        click.secho("Radio {} do NOT exists!".format(radio_id), fg="red")


@radio.command(help="Play a radio.", no_args_is_help=True)
@click.argument("radio-id", required=True)
def play(radio_id):
    radios = load_radios()
    if radio_id not in radios:
        click.secho("Radio {} do NOT exists!".format(radio_id), fg="red")
        sys.exit(1)

    click.secho("Playing {name}!".format(**radios[radio_id]), bold=True)
    play_url(radios[radio_id]["url"])


@radio.command(help="Show all radios information.")
@click.option("--urls", is_flag=True, help="Also show Streaming URLS.")
@click.option("--count", is_flag=True, help="Show how many radios are available.")
def show(urls, count):
    radios = load_radios()
    show_list(radios, urls, count)


@radio.command(help="Search radio in the available radios.", no_args_is_help=True)
@click.option("-i", "--invert", is_flag=True, help="Invert filter.")
@click.argument("string", required=True)
def search(string, invert):
    radios = load_radios()
    show_list(radios, str_filter=string.lower(), invert=invert)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['radio']

package_data = \
{'': ['*'], 'radio': ['data/*']}

install_requires = \
['click>=7,<8']

entry_points = \
{'console_scripts': ['radio = radio.cli:radio']}

setup_kwargs = {
    'name': 'radio',
    'version': '0.1.3',
    'description': 'Just listen to the radio',
    'long_description': '=====\nradio\n=====\n\n**radio** is a command line tool to **just listen to the radio**.\n\n.. contents::\n\nInstallation\n============\n\n``pip install radio``\n\nor\n\n``pip install --upgrade radio``\n\nUsage\n=====\n\nJust play the radio like this...\n\n``radio play <id>``\n\nLook for available radios with\n\n``radio show``\n\nSearch for you favourite radio with\n\n``radio search <term>``\n\nor else add it with\n\n``radio add <id> --name "My Favourite Radio" --url <streaming-url>``\n\nGeneral usage:\n\n``radio [OPTIONS] COMMAND [ARGS]...``\n\nOptions:\n\n--version  Show the version and exit.\n--help     Show this message and exit.\n\nCommands\n========\n\n:add:     Add or update a radio information.\n:play:    Play a radio.\n:remove:  Remove a radio information.\n:search:  Search radio in the available radios.\n:show:    Show all radios information.\n\nadd\n---\n\nAdd or update a radio information.\n\nUsage:\n\n``radio add [OPTIONS] RADIO_ID``\n\nOptions:\n\n-n, --name TEXT  Radio complete fancy name.  [required]\n-u, --url TEXT   Radio playable streaming url.  [required]\n--help           Show this message and exit.\n\nFor example::\n\n    radio add convos --name "Radio Con Vos FM 89.9" --url https://server1.stweb.tv/rcvos/live/chunks.m3u8\n\nplay\n----\n\nPlay a radio.\n\nUsage:\n\n``radio play [OPTIONS] RADIO_ID``\n\nOptions:\n\n--help  Show this message and exit.\n    \n*Turn off* the radio by pressing "q" or with Ctrl-<C>.\n\nremove\n------\n\nRemove a radio information.\n\nUsage:\n\n``radio remove [OPTIONS] RADIO_ID``\n\nOptions:\n\n--help  Show this message and exit.\n\nsearch\n------\n\nSearch radio in the available radios.\n\nUsage:\n\n``radio search [OPTIONS] STRING``\n\nOptions:\n\n-i, --invert  Invert filter.\n--help        Show this message and exit.\n\nshow\n----\n\nShow all radios information.\n\nUsage:\n\n``radio show [OPTIONS]``\n\nOptions:\n\n--urls   Also show Streaming URLS.\n--count  Show how many radios are available.\n--help   Show this message and exit\n\nHow does it *plays* the radio?\n==============================\n\nIt requires any of the following media player:\n\n- **ffplayer** (ffmpeg package)\n- **cvlc** (vlc package)\n- **mplayer**\n\nPriority or alternative players yet to make *customizable* in future versions.\n\nToDo\n====\n\n- support multiples radio lists (is it useful?)\n- support downloading radio lists from somewhere\n- customize player and priorities or autodetect (something like *rifle* in the *ranger-fm* package)\n- what more?\n- help me at https://github.com/quijot/radio-package\n\nAuthor\n======\n\n* `quijoT <https://github.com/quijot>`_ (Santiago Pestarini <santiagonob@gmail.com>)\n\nCollaborators\n-------------\n\n* `sdeancos <https://github.com/sdeancos>`_ (Samuel de Ancos)\n\nLicense\n=======\n\nradio is licensed under the *do What The Fuck you want to Public License*, WTFPL. See the LICENSE file.\n',
    'author': 'Santiago Pestarini',
    'author_email': 'santiagonob@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/quijot/radio-package',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
